@@ -1,12 +1,18 @@
 """
 Example of simple consumer that waits for a single message, acknowledges it
 and exits.
+
 """
-from kombu import Connection, Exchange, Queue, Consumer, eventloop
+from __future__ import absolute_import, unicode_literals, print_function
+
 from pprint import pformat
 
+from kombu import Connection, Exchange, Queue, Consumer, eventloop
+
 #: By default messages sent to exchanges are persistent (delivery_mode=2),
 #: and queues and exchanges are durable.
+exchange = Exchange('kombu_demo', type='direct')
+queue = Queue('kombu_demo', exchange, routing_key='kombu_demo')
 
 
 def pretty(obj):
@@ -15,25 +21,16 @@ def pretty(obj):
 
 #: This is the callback applied when a message is received.
 def handle_message(body, message):
-    print('Received message: %r' % (body, ))
-    print('  properties:\n%s' % (pretty(message.properties), ))
-    print('  delivery_info:\n%s' % (pretty(message.delivery_info), ))
+    print('Received message: {0!r}'.format(body))
+    print('  properties:\n{0}'.format(pretty(message.properties)))
+    print('  delivery_info:\n{0}'.format(pretty(message.delivery_info)))
     message.ack()
 
 #: Create a connection and a channel.
 #: If hostname, userid, password and virtual_host is not specified
 #: the values below are the default, but listed here so it can
 #: be easily changed.
-with Connection('pyamqp://guest:guest@localhost:5672//') as connection:
-    # The configuration of the message flow is as follows:
-    #   gateway_kombu_exchange -> internal_kombu_exchange -> kombu_demo queue
-    gateway_exchange = Exchange('gateway_kombu_demo')(connection)
-    exchange = Exchange('internal_kombu_demo')(connection)
-    gateway_exchange.declare()
-    exchange.declare()
-    exchange.bind_to(gateway_exchange, routing_key='kombu_demo')
-
-    queue = Queue('kombu_demo', exchange, routing_key='kombu_demo')
+with Connection('amqp://guest:guest@localhost:5672//') as connection:
 
     #: Create consumer using our callback and queue.
     #: Second argument can also be a list to consume from
