@@ -2,7 +2,7 @@
 from __future__ import absolute_import, unicode_literals
 
 from kombu import Connection, Exchange, Queue, Producer, Consumer
-from kombu.async import Hub
+from kombu.asynchronous import Hub
 
 hub = Hub()
 exchange = Exchange('asynt')
