@@ -1,11 +1,5 @@
-"""
-kombu.serialization
-===================
-
-Serialization utilities.
-
-"""
-from __future__ import absolute_import
+"""Serialization utilities."""
+from __future__ import absolute_import, unicode_literals
 
 import codecs
 import os
@@ -19,13 +13,14 @@ except ImportError:  # pragma: no cover
 
 from collections import namedtuple
 from contextlib import contextmanager
+from io import BytesIO
 
 from .exceptions import (
     ContentDisallowed, DecodeError, EncodeError, SerializerNotInstalled
 )
-from .five import BytesIO, reraise, text_t
-from .utils import entrypoints
-from .utils.encoding import str_to_bytes, bytes_t
+from .five import reraise, text_t
+from .utils.compat import entrypoints
+from .utils.encoding import bytes_to_str, str_to_bytes, bytes_t
 
 __all__ = ['pickle', 'loads', 'dumps', 'register', 'unregister']
 SKIP_DECODE = frozenset(['binary', 'ascii-8bit'])
@@ -50,7 +45,7 @@ codec = namedtuple('codec', ('content_type', 'content_encoding', 'encoder'))
 
 @contextmanager
 def _reraise_errors(wrapper,
-                    include=(Exception, ), exclude=(SerializerNotInstalled, )):
+                    include=(Exception,), exclude=(SerializerNotInstalled,)):
     try:
         yield
     except exclude:
@@ -83,6 +78,28 @@ class SerializerRegistry(object):
 
     def register(self, name, encoder, decoder, content_type,
                  content_encoding='utf-8'):
+        """Register a new encoder/decoder.
+
+        Arguments:
+            name (str): A convenience name for the serialization method.
+
+            encoder (callable): A method that will be passed a python data
+                structure and should return a string representing the
+                serialized data.  If :const:`None`, then only a decoder
+                will be registered. Encoding will not be possible.
+
+            decoder (Callable): A method that will be passed a string
+                representing serialized data and should return a python
+                data structure.  If :const:`None`, then only an encoder
+                will be registered.  Decoding will not be possible.
+
+            content_type (str): The mime-type describing the serialized
+                structure.
+
+            content_encoding (str): The content encoding (character set) that
+                the `decoder` method will be returning. Will usually be
+                `utf-8`, `us-ascii`, or `binary`.
+        """
         if encoder:
             self._encoders[name] = codec(
                 content_type, content_encoding, encoder,
@@ -103,6 +120,15 @@ class SerializerRegistry(object):
         self._disabled_content_types.add(name)
 
     def unregister(self, name):
+        """Unregister registered encoder/decoder.
+
+        Arguments:
+            name (str): Registered serialization method name.
+
+        Raises:
+            SerializerNotInstalled: If a serializer by that name
+                cannot be found.
+        """
         try:
             content_type = self.name_to_type[name]
             self._decoders.pop(content_type, None)
@@ -114,15 +140,16 @@ class SerializerRegistry(object):
                 'No encoder/decoder installed for {0}'.format(name))
 
     def _set_default_serializer(self, name):
-        """
-        Set the default serialization method used by this library.
+        """Set the default serialization method used by this library.
 
-        :param name: The name of the registered serialization method.
-            For example, `json` (default), `pickle`, `yaml`, `msgpack`,
-            or any custom methods registered using :meth:`register`.
+        Arguments:
+            name (str): The name of the registered serialization method.
+                For example, `json` (default), `pickle`, `yaml`, `msgpack`,
+                or any custom methods registered using :meth:`register`.
 
-        :raises SerializerNotInstalled: If the serialization method
-            requested is not available.
+        Raises:
+            SerializerNotInstalled: If the serialization method
+                requested is not available.
         """
         try:
             (self._default_content_type, self._default_content_encoding,
@@ -132,6 +159,36 @@ class SerializerRegistry(object):
                 'No encoder installed for {0}'.format(name))
 
     def dumps(self, data, serializer=None):
+        """Encode data.
+
+        Serialize a data structure into a string suitable for sending
+        as an AMQP message body.
+
+        Arguments:
+            data (List, Dict, str): The message data to send.
+
+            serializer (str): An optional string representing
+                the serialization method you want the data marshalled
+                into. (For example, `json`, `raw`, or `pickle`).
+
+                If :const:`None` (default), then json will be used, unless
+                `data` is a :class:`str` or :class:`unicode` object. In this
+                latter case, no serialization occurs as it would be
+                unnecessary.
+
+                Note that if `serializer` is specified, then that
+                serialization method will be used even if a :class:`str`
+                or :class:`unicode` object is passed in.
+
+        Returns:
+            Tuple[str, str, str]: A three-item tuple containing the
+            content type (e.g., `application/json`), content encoding, (e.g.,
+            `utf-8`) and a string containing the serialized data.
+
+        Raises:
+            SerializerNotInstalled: If the serialization method
+                requested is not available.
+        """
         if serializer == 'raw':
             return raw_encode(data)
         if serializer and not self._encoders.get(serializer):
@@ -163,11 +220,33 @@ class SerializerRegistry(object):
         with _reraise_errors(EncodeError):
             payload = encoder(data)
         return content_type, content_encoding, payload
-    encode = dumps  # XXX compat
 
     def loads(self, data, content_type, content_encoding,
               accept=None, force=False, _trusted_content=TRUSTED_CONTENT):
-        content_type = content_type or 'application/data'
+        """Decode serialized data.
+
+        Deserialize a data stream as serialized using `dumps`
+        based on `content_type`.
+
+        Arguments:
+            data (bytes, buffer, str): The message data to deserialize.
+
+            content_type (str): The content-type of the data.
+                (e.g., `application/json`).
+
+            content_encoding (str): The content-encoding of the data.
+                (e.g., `utf-8`, `binary`, or `us-ascii`).
+
+            accept (Set): List of content-types to accept.
+
+        Raises:
+            ContentDisallowed: If the content-type is not accepted.
+
+        Returns:
+            Any: The unserialized data.
+        """
+        content_type = (bytes_to_str(content_type) if content_type
+                        else 'application/data')
         if accept is not None:
             if content_type not in _trusted_content \
                     and content_type not in accept:
@@ -187,7 +266,6 @@ class SerializerRegistry(object):
                 with _reraise_errors(DecodeError):
                     return _decode(data, content_encoding)
         return data
-    decode = loads  # XXX compat
 
     def _for_untrusted_content(self, ctype, why):
         return ContentDisallowed(
@@ -200,95 +278,9 @@ class SerializerRegistry(object):
 
 #: Global registry of serializers/deserializers.
 registry = SerializerRegistry()
-
-
-"""
-.. function:: dumps(data, serializer=default_serializer)
-
-    Serialize a data structure into a string suitable for sending
-    as an AMQP message body.
-
-    :param data: The message data to send. Can be a list,
-        dictionary or a string.
-
-    :keyword serializer: An optional string representing
-        the serialization method you want the data marshalled
-        into. (For example, `json`, `raw`, or `pickle`).
-
-        If :const:`None` (default), then json will be used, unless
-        `data` is a :class:`str` or :class:`unicode` object. In this
-        latter case, no serialization occurs as it would be
-        unnecessary.
-
-        Note that if `serializer` is specified, then that
-        serialization method will be used even if a :class:`str`
-        or :class:`unicode` object is passed in.
-
-    :returns: A three-item tuple containing the content type
-        (e.g., `application/json`), content encoding, (e.g.,
-        `utf-8`) and a string containing the serialized
-        data.
-
-    :raises SerializerNotInstalled: If the serialization method
-            requested is not available.
-"""
-dumps = encode = registry.encode   # XXX encode is a compat alias
-
-"""
-.. function:: loads(data, content_type, content_encoding):
-
-    Deserialize a data stream as serialized using `dumps`
-    based on `content_type`.
-
-    :param data: The message data to deserialize.
-
-    :param content_type: The content-type of the data.
-        (e.g., `application/json`).
-
-    :param content_encoding: The content-encoding of the data.
-        (e.g., `utf-8`, `binary`, or `us-ascii`).
-
-    :returns: The unserialized data.
-
-"""
-loads = decode = registry.decode  # XXX decode is a compat alias
-
-
-"""
-.. function:: register(name, encoder, decoder, content_type,
-                       content_encoding='utf-8'):
-    Register a new encoder/decoder.
-
-    :param name: A convenience name for the serialization method.
-
-    :param encoder: A method that will be passed a python data structure
-        and should return a string representing the serialized data.
-        If :const:`None`, then only a decoder will be registered. Encoding
-        will not be possible.
-
-    :param decoder: A method that will be passed a string representing
-        serialized data and should return a python data structure.
-        If :const:`None`, then only an encoder will be registered.
-        Decoding will not be possible.
-
-    :param content_type: The mime-type describing the serialized
-        structure.
-
-    :param content_encoding: The content encoding (character set) that
-        the `decoder` method will be returning. Will usually be
-        `utf-8`, `us-ascii`, or `binary`.
-
-"""
+dumps = registry.dumps
+loads = registry.loads
 register = registry.register
-
-
-"""
-.. function:: unregister(name):
-    Unregister registered encoder/decoder.
-
-    :param name: Registered serialization method name.
-
-"""
 unregister = registry.unregister
 
 
@@ -307,14 +299,9 @@ def raw_encode(data):
 
 def register_json():
     """Register a encoder/decoder for JSON serialization."""
-    from anyjson import loads as json_loads, dumps as json_dumps
+    from kombu.utils import json as _json
 
-    def _loads(obj):
-        if isinstance(obj, bytes_t):
-            obj = obj.decode()
-        return json_loads(obj)
-
-    registry.register('json', json_dumps, _loads,
+    registry.register('json', _json.dumps, _json.loads,
                       content_type='application/json',
                       content_encoding='utf-8')
 
@@ -323,7 +310,9 @@ def register_yaml():
     """Register a encoder/decoder for YAML serialization.
 
     It is slower than JSON, but allows for more data types
-    to be serialized. Useful if you need to send data such as dates"""
+    to be serialized. Useful if you need to send data such as dates
+
+    """
     try:
         import yaml
         registry.register('yaml', yaml.safe_dump, yaml.safe_load,
@@ -332,8 +321,11 @@ def register_yaml():
     except ImportError:
 
         def not_available(*args, **kwargs):
-            """In case a client receives a yaml message, but yaml
-            isn't installed."""
+            """Raise SerializerNotInstalled.
+
+            Used in case a client receives a yaml message, but yaml
+            isn't installed.
+            """
             raise SerializerNotInstalled(
                 'No decoder installed for YAML. Install the PyYAML library')
         registry.register('yaml', None, not_available, 'application/x-yaml')
@@ -349,9 +341,11 @@ else:
 
 
 def register_pickle():
-    """The fastest serialization method, but restricts
-    you to python clients."""
+    """Register pickle serializer.
 
+    The fastest serialization method, but restricts
+    you to python clients.
+    """
     def pickle_dumps(obj, dumper=pickle.dumps):
         return dumper(obj, protocol=pickle_protocol)
 
@@ -361,30 +355,39 @@ def register_pickle():
 
 
 def register_msgpack():
-    """See http://msgpack.sourceforge.net/"""
+    """Register msgpack serializer.
+
+    See Also:
+        https://msgpack.org/.
+    """
+    pack = unpack = None
     try:
-        try:
-            from msgpack import packb as pack, unpackb
+        import msgpack
+        if msgpack.version >= (0, 4):
+            from msgpack import packb, unpackb
 
-            def unpack(s, encoding='utf-8'):
-                return unpackb(s, encoding=encoding)
-        except ImportError:
-            # msgpack < 0.2.0 and Python 2.5
-            from msgpack import packs as pack, unpacks as unpack  # noqa
-        registry.register(
-            'msgpack', pack, unpack,
-            content_type='application/x-msgpack',
-            content_encoding='binary')
+            def pack(s):
+                return packb(s, use_bin_type=True)
+
+            def unpack(s):
+                return unpackb(s, encoding='utf-8')
+        else:
+            def version_mismatch(*args, **kwargs):
+                raise SerializerNotInstalled(
+                    'msgpack requires msgpack-python >= 0.4.0')
+            pack = unpack = version_mismatch
     except (ImportError, ValueError):
-
         def not_available(*args, **kwargs):
-            """In case a client receives a msgpack message, but msgpack
-            isn't installed."""
             raise SerializerNotInstalled(
                 'No decoder installed for msgpack. '
-                'Please install the msgpack library')
-        registry.register('msgpack', None, not_available,
-                          'application/x-msgpack')
+                'Please install the msgpack-python library')
+        pack = unpack = not_available
+    registry.register(
+        'msgpack', pack, unpack,
+        content_type='application/x-msgpack',
+        content_encoding='binary',
+    )
+
 
 # Register the base serialization methods.
 register_json()
@@ -411,10 +414,10 @@ _setupfuns = {
 def enable_insecure_serializers(choices=['pickle', 'yaml', 'msgpack']):
     """Enable serializers that are considered to be unsafe.
 
-    Will enable ``pickle``, ``yaml`` and ``msgpack`` by default,
-    but you can also specify a list of serializers (by name or content type)
-    to enable.
-
+    Note:
+        Will enable ``pickle``, ``yaml`` and ``msgpack`` by default, but you
+        can also specify a list of serializers (by name or content type)
+        to enable.
     """
     for choice in choices:
         try:
@@ -429,12 +432,10 @@ def disable_insecure_serializers(allowed=['json']):
     Will disable all serializers except ``json``
     or you can specify a list of deserializers to allow.
 
-    .. note::
-
+    Note:
         Producers will still be able to serialize data
         in these formats, but consumers will not accept
         incoming data using the untrusted content types.
-
     """
     for name in registry._decoders:
         registry.disable(name)
@@ -453,5 +454,5 @@ for ep, args in entrypoints('kombu.serializers'):  # pragma: no cover
 
 def prepare_accept_content(l, name_to_type=registry.name_to_type):
     if l is not None:
-        return set(n if '/' in n else name_to_type[n] for n in l)
+        return {n if '/' in n else name_to_type[n] for n in l}
     return l
