@@ -1,39 +1,29 @@
+"""`librabbitmq`_ transport.
+
+.. _`librabbitmq`: https://pypi.python.org/librabbitmq/
 """
-kombu.transport.librabbitmq
-===========================
-
-`librabbitmq`_ transport.
-
-.. _`librabbitmq`: http://pypi.python.org/librabbitmq/
-
-"""
-from __future__ import absolute_import
+from __future__ import absolute_import, unicode_literals
 
 import os
 import socket
 import warnings
 
-try:
-    import librabbitmq as amqp
-    from librabbitmq import ChannelError, ConnectionError
-except ImportError:  # pragma: no cover
-    try:
-        import pylibrabbitmq as amqp                             # noqa
-        from pylibrabbitmq import ChannelError, ConnectionError  # noqa
-    except ImportError:
-        raise ImportError('No module named librabbitmq')
+import librabbitmq as amqp
+from librabbitmq import ChannelError, ConnectionError
 
 from kombu.five import items, values
 from kombu.utils.amq_manager import get_manager
 from kombu.utils.text import version_string_as_tuple
 
 from . import base
+from .base import to_rabbitmq_queue_arguments
 
 W_VERSION = """
     librabbitmq version too old to detect RabbitMQ version information
     so make sure you are using librabbitmq 1.5 when using rabbitmq > 3.3
 """
 DEFAULT_PORT = 5672
+DEFAULT_SSL_PORT = 5671
 
 NO_SSL_ERROR = """\
 ssl not supported by librabbitmq, please use pyamqp:// or stunnel\
@@ -41,10 +31,11 @@ ssl not supported by librabbitmq, please use pyamqp:// or stunnel\
 
 
 class Message(base.Message):
+    """AMQP Message (librabbitmq)."""
 
     def __init__(self, channel, props, info, body):
         super(Message, self).__init__(
-            channel,
+            channel=channel,
             body=body,
             delivery_info=info,
             properties=props,
@@ -55,6 +46,8 @@ class Message(base.Message):
 
 
 class Channel(amqp.Channel, base.StdChannel):
+    """AMQP Channel (librabbitmq)."""
+
     Message = Message
 
     def prepare_message(self, body, priority=None,
@@ -68,31 +61,46 @@ class Channel(amqp.Channel, base.StdChannel):
                            'priority': priority})
         return body, properties
 
+    def prepare_queue_arguments(self, arguments, **kwargs):
+        arguments = to_rabbitmq_queue_arguments(arguments, **kwargs)
+        return {k.encode('utf8'): v for k, v in items(arguments)}
+
 
 class Connection(amqp.Connection):
+    """AMQP Connection (librabbitmq)."""
+
     Channel = Channel
     Message = Message
 
 
 class Transport(base.Transport):
+    """AMQP Transport (librabbitmq)."""
+
     Connection = Connection
 
     default_port = DEFAULT_PORT
+    default_ssl_port = DEFAULT_SSL_PORT
+
     connection_errors = (
         base.Transport.connection_errors + (
             ConnectionError, socket.error, IOError, OSError)
     )
     channel_errors = (
-        base.Transport.channel_errors + (ChannelError, )
+        base.Transport.channel_errors + (ChannelError,)
     )
     driver_type = 'amqp'
     driver_name = 'librabbitmq'
 
-    supports_ev = True
+    implements = base.Transport.implements.extend(
+        asynchronous=True,
+        heartbeats=False,
+    )
 
     def __init__(self, client, **kwargs):
         self.client = client
         self.default_port = kwargs.get('default_port') or self.default_port
+        self.default_ssl_port = (kwargs.get('default_ssl_port') or
+                                 self.default_ssl_port)
         self.__reader = None
 
     def driver_version(self):
@@ -138,7 +146,7 @@ class Transport(base.Transport):
                 channel.connection = None
             try:
                 os.close(connection.fileno())
-            except OSError:
+            except (OSError, ValueError):
                 pass
             connection.channels.clear()
             connection.callbacks.clear()
@@ -168,6 +176,11 @@ class Transport(base.Transport):
 
     @property
     def default_connection_params(self):
-        return {'userid': 'guest', 'password': 'guest',
-                'port': self.default_port,
-                'hostname': 'localhost', 'login_method': 'AMQPLAIN'}
+        return {
+            'userid': 'guest',
+            'password': 'guest',
+            'port': (self.default_ssl_port if self.client.ssl
+                     else self.default_port),
+            'hostname': 'localhost',
+            'login_method': 'AMQPLAIN',
+        }
