@@ -211,7 +211,7 @@ class Channel(redis.Channel):
     def _get_client(self):
         return Client
 
-    def _get_pool(self, async=False):
+    def _get_pool(self, asynchronous=False):
         return Mock()
 
     def _get_response_error(self):
